﻿var port = chrome.runtime.connect();
port.onMessage.addListener(function(message) {
    message.messageFromContentScript1234 = true
    window.postMessage(message, '*');
});
window.cameraLocalStream = window.cameraLocalStream ? window.cameraLocalStream : false;

window.addEventListener('message', function(event) {
    if (event.source != window || !event.data.messageFromContentScript1234) return;
    port.postMessage(event.data);
});

// ctrl+tab to stop the recording
var isControlKeyPressed;
window.addEventListener('keydown', function(e) {
    keyCode = e.which || e.keyCode || 0;

    if(keyCode === 17) {
        isControlKeyPressed = true;
    }
}, false);

window.addEventListener('keyup', function(e) {
    keyCode = e.which || e.keyCode || 0;

    if(isControlKeyPressed && keyCode === 32) {
        port.postMessage({
            messageFromContentScript1234: true,
            stopRecording: true,
            dropdown: true
        });

        e.preventDefault();
        e.stopPropagation();
        return false;
    }

    if(keyCode === 17) {
        isControlKeyPressed = false;
    }
}, false);

function RecordRTC_Extension(config) {
    config = config || {
        enableTabCaptureAPI: false,
        enableTabCaptureAPIAudioOnly: false,
        enableScreen: false,
        enableSpeakers: false,
        enableCamera: false,
        enableMicrophone: false
    };

    var startCallback = function() {
        //
    };

    var stopCallback = function() {
        //
    };

    var supportedValues = ['enableTabCaptureAPI', 'enableTabCaptureAPIAudioOnly', 'enableScreen', 'enableSpeakers', 'enableCamera', 'enableMicrophone'];

    window.addEventListener('message', function(event) {
        if (event.source != window || !event.data.messageFromContentScript1234) return;

        if(event.data.startedRecording === true) {
            startCallback();
        }

        if(event.data.stoppedRecording === true) {
            stopCallback(dataURItoBlob(event.data.file));
        }
    });

    // via: https://stackoverflow.com/a/12300351/552182
    function dataURItoBlob(dataURI) {
      var byteString = atob(dataURI.split(',')[1]);
      var mimeString = dataURI.split(',')[0].split(':')[1].split(';')[0]
      var ab = new ArrayBuffer(byteString.length);
      var ia = new Uint8Array(ab);
      for (var i = 0; i < byteString.length; i++) {
          ia[i] = byteString.charCodeAt(i);
      }
      var blob = new Blob([ab], {type: mimeString});
      return blob;
    }

    function getSupoortedFormats() {
        return [{
            enableScreen: true
        }, {
            enableScreen: true,
            enableMicrophone: true
        }, {
            enableScreen: true,
            enableSpeakers: true
        }, {
            enableScreen: true,
            enableMicrophone: true,
            enableSpeakers: true
        }, {
            enableTabCaptureAPI: true
        }, {
            enableTabCaptureAPI: true,
            enableTabCaptureAPIAudioOnly: true
        }, {
            enableScreen: true,
            enableCamera: true
        }, {
            enableMicrophone: true,
            enableCamera: true
        }, {
            enableMicrophone: true,
            enableSpeakers: true
        }, {
            enableMicrophone: true
        }, {
            enableSpeakers: true
        }];
    }

    return {
        startRecording: function(options, callback) {
            startCallback = callback || function() {};

            if(options) {
                if(typeof options != 'object') {
                    callback(false, 'First parameter must be an object.')
                    return;
                }

                if(!Object.keys(options).length) {
                    callback(false, 'First parameter must specify at least one recoding type.')
                    return;
                }

                var mismatchedValues = [];
                Object.keys(options).forEach(function(key) {
                    if(supportedValues.indexOf(key) == -1) {
                        mismatchedValues.push(key);
                    }
                });

                if(mismatchedValues.length) {
                    callback(false, 'Unsupported parameters detected: ' + mismatchedValues.join(', '))
                    return;
                }

                config = options; // override
            }

            config.messageFromContentScript1234 = true;
            config.startRecording = true;
            config.RecordRTC_Extension = true; // hint for background page

            window.postMessage(config, '*');
        },
        stopRecording: function(callback) {
            stopCallback = callback || function() {};

            if(!config.startRecording) {
                callback(false, 'There is no recorder to stop.');
                return;
            }

            config.startRecording = false;
            config.stopRecording = true;

            window.postMessage(config, '*');
        },
        getSupoortedFormats: getSupoortedFormats
    }
}


function handleWebcamPreview(close) {
    var handleSuccess = function(stream) {
       webcamPreview = document.getElementById('webcam-preview');
       window.cameraLocalStream = stream;
       webcamPreview.innerHTML = '<div style="width: 100%; height: 100%; border-radius: 50%; border: 2px solid #e3e3e3; background-color: #fff; overflow: hidden; box-shadow: 0px 0px 20px 6px #33333355;">' +
                                 '<video style="transform: scale(1.4);" id="webcam-video" width="200" height="200" autoplay></video></div>';
       webcamPreview.style.position = 'fixed';
       webcamPreview.style.bottom = '4px';
       webcamPreview.style.left = '4px';
       webcamPreview.style.zIndex = 9999999;

       document.getElementById('webcam-video').srcObject = stream;
       document.getElementById('webcam-video').play();
    };

    if (close && document.getElementById('webcam-preview')) {
        var elem = document.getElementById('webcam-preview');

        elem.parentNode.removeChild(elem);
        window.webcamPreviewEnabled = false;

        if (window.cameraLocalStream) {
            window.cameraLocalStream.getTracks()[0].stop();
        }

        return;
    }
    if (!close && !window.webcamPreviewEnabled) {
        window.webcamPreviewEnabled = true;
        var webcamPreview = document.createElement('div');

        webcamPreview.setAttribute("id", "webcam-preview");
        (document.body || document.documentElement).appendChild(webcamPreview);
        navigator.mediaDevices.getUserMedia({
            audio: false,
            video: true
        })
        .then(handleSuccess);
    }
}
function toggleCountdown(hide) {
    if (!hide && !window.countdownVisible) {
        window.countdownVisible = true;
        var countdownWrapper = document.createElement('div');
        var countdown = document.createElement('div');

        countdownWrapper.setAttribute("id", "countdown-wrapper");
        countdownWrapper.style.position = 'fixed';
        countdownWrapper.style.top = '0';
        countdownWrapper.style.left = '0';
        countdownWrapper.style.width = '100%';
        countdownWrapper.style.height = '100%';
        countdownWrapper.style.display = 'flex';
        countdownWrapper.style.alignItems = 'center';
        countdownWrapper.style.justifyContent = 'center';
        countdownWrapper.style.backgroundColor = 'rgba(0, 0, 0, 0.66)';
        countdownWrapper.style.zIndex = 9999999;

        countdown.innerHTML = '4';
        countdown.setAttribute("id", "countdown");
        countdown.style.fontSize = "180px";
        countdown.style.textShadow = "2px -10px 40px black";
        countdown.style.fontWeight = "600";
        countdown.style.transition = "all 0.4s";
        countdown.style.color = "#fff";
        countdown.style.marginLeft = "35px";

        (document.body || document.documentElement).appendChild(countdownWrapper);
        countdownWrapper.appendChild(countdown);

        var currentTime = 4;

        var countdownInterval = setInterval(function() {
            var countdown = document.getElementById('countdown');
            var color = '#e3e3e3';
            currentTime --;

            switch (currentTime) {
                case 3:
                    color = '#868200';
                    break;
                case 2:
                    color = '#865d00';
                    break;
                case 1:
                    color = '#a04200';
                    break;
                case 0:
                    color = '#c12400';
                    break;
            }

            countdown.innerText = currentTime;
            //countdown.style.color = color;
            countdown.style.color = '#fff';

            if (currentTime === 0) {
                setTimeout(function() {
                    clearInterval(countdownInterval);
                    window.countdownVisible = false;
                    var countdown = document.getElementById('countdown-wrapper');

                    countdown.parentNode.removeChild(countdown);
                }, 300);
            }
        }, 1100);
    }
}

chrome.runtime.onMessage.addListener( function(request, sender, sendResponse) {
    if (request && request.data === "startCountdown") {
        toggleCountdown();
    }
    if (request && request.data === 'webcamPreview-stop') {
        handleWebcamPreview(true);
    }
    if (request && request.data === 'webcamPreview') {
        handleWebcamPreview();
    }
});
var existingScript = document.querySelectorAll('[name="handleWebcamPreview_vooPlayerExtension"]');
var initialScript = document.createElement('script');

if (!existingScript) {
    initialScript.setAttribute('name', 'handleWebcamPreview_vooPlayerExtension');
    (document.head || document.body || document.documentElement).appendChild(initialScript).text = handleWebcamPreview.toString() + ';';
}
var script = document.createElement('script');
(document.head || document.body || document.documentElement).appendChild(script).text = RecordRTC_Extension.toString() + ';';
script.remove();
