var runtimePort;

chrome.runtime.onConnect.addListener(function(port) {
    runtimePort = port;

    runtimePort.onMessage.addListener(function(message) {
        if (message && message.openSignUp) {
            openSignUp();

            return;
        }
        if (message && message.resetControls) {
            if (isRecording) {
                console.log('Stop recording...');
                stopScreenRecording();
            }
            chrome.storage.sync.set({
                enableTabCaptureAPI: 'false',
                enableTabCaptureAPIAudioOnly: 'false',
                enableMicrophone: 'true',
                enableCamera: 'true',
                enableScreen: 'true',
                enableSpeakers: 'false',
                isRecording: 'false'
            }, function() {
                getUserConfigs();
            });

            return;
        }
        if (message && message.startRecording) {
            console.log('Start recording...');
            chrome.storage.sync.set({
                isRecording: 'true'
            }, function() {
                getUserConfigs(true);
            });

            return;
        }
        if (message && message.stopRecording) {
            console.log('Stop recording...');

            if (isRecording) {
                stopScreenRecording();
                chrome.storage.sync.set({
                    isRecording: 'false'
                }, function() {
                    getUserConfigs();
                });
            }

            return;
        }
        if (message && message.controlSwitch) {
            console.log('Control switch -- ', message.controlSwitch);
            var storageObject = {
                isRecording: 'false'
            };

            switch (message.controlSwitch.name) {
                case 'camera':
                    storageObject.enableCamera = String(message.controlSwitch.value);

                    break;
                case 'screen':
                    storageObject.enableScreen = String(message.controlSwitch.value);

                    break;
                case 'mic':
                    storageObject.enableMicrophone = String(message.controlSwitch.value);

                    break;
                case 'camera-box':
                    storageObject.previewCameraBoxMode = String(message.controlSwitch.value);

                    break;
            }

            chrome.storage.sync.set(storageObject, function() {
                getUserConfigs();
            });

            return;
        }
        if (!message || !message.messageFromContentScript1234) {
            return;
        }

        if (message.startRecording) {
            if(message.dropdown) {
                openPreviewOnStopRecording = true;
                openCameraPreviewDuringRecording = true;
            }

            if (isRecording && message.dropdown) {
                stopScreenRecording();
                return;
            }

            if(message.RecordRTC_Extension) {
                openPreviewOnStopRecording = false;
                openCameraPreviewDuringRecording = false;
                
                enableTabCaptureAPI = message['enableTabCaptureAPI'] === true;
                enableTabCaptureAPIAudioOnly = message['enableTabCaptureAPIAudioOnly'] === true;
                enableScreen = message['enableScreen'] === true;
                enableMicrophone = message['enableMicrophone'] === true;
                enableCamera = message['enableCamera'] === true;
                enableSpeakers = message['enableSpeakers'] === true;

                startRecordingCallback = function(file) {
                    port.postMessage({
                        messageFromContentScript1234: true,
                        startedRecording: true
                    });
                };

                chrome.storage.sync.set({
                    enableTabCaptureAPI: enableTabCaptureAPI ? 'true' : 'false',
                    enableTabCaptureAPIAudioOnly: enableTabCaptureAPIAudioOnly ? 'true' : 'false',
                    enableMicrophone: enableMicrophone ? 'true' : 'false',
                    enableCamera: enableCamera ? 'true' : 'false',
                    enableScreen: enableScreen ? 'true' : 'false',
                    enableSpeakers: enableSpeakers ? 'true' : 'false',
                    isRecording: 'true'
                }, function() {
                    getUserConfigs();
                });
                return;
            }

            getUserConfigs();
            return;
        }

        if (message.stopRecording) {
            if(message.RecordRTC_Extension) {
                stopRecordingCallback = function(file) {
                    var reader = new FileReader();
                    reader.onload = function(e) {
                        port.postMessage({
                            messageFromContentScript1234: true,
                            stoppedRecording: true,
                            file: e.target.result
                        });
                    };
                    reader.readAsDataURL(file);
                };
            }

            stopScreenRecording();
            return;
        }
    });
});
